# Generated by Django 2.0.2 on 2018-02-13 15:10

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('slackchat', '0001_squashed_0010_auto_20180210_1726'),
    ]

    operations = [
        migrations.AddField(
            model_name='channel',
            name='team_id',
            field=models.SlugField(blank=True, editable=False, help_text='Slack API team ID', max_length=10, null=True),
        ),
        migrations.AlterField(
            model_name='channel',
            name='api_id',
            field=models.SlugField(blank=True, editable=False, help_text='Slack API channel ID', max_length=10, null=True),
        ),
    ]



import re

from django.db import models
from django.utils.safestring import mark_safe
from markdown import markdown

from .custom_content_template import CustomContentTemplate


class Message(models.Model):
    """
    A message posted in a Slack channel by a user.
    """

    timestamp = models.DateTimeField(unique=True)

    channel = models.ForeignKey(
        'Channel', related_name='messages', on_delete=models.CASCADE)
    user = models.ForeignKey(
        'User', related_name='messages', on_delete=models.CASCADE)
    text = models.TextField()

    def html(self):
        return mark_safe(markdown(self.get_content()))

    def get_content(self):
        template, match = self.find_template_match()
        if match:
            groups = [group for group in match.groups()]
            return template.content_template.format(*groups)
        return self.text

    def find_template_match(self):
        for template in CustomContentTemplate.objects.filter(
            chat_type=self.channel.chat_type
        ):
            match = re.search(template.search_string, self.text)
            if match:
                return (template, match)
        return (None, None)

    def __str__(self):
        return self.text[:50]



# Generated by Django 2.0.2 on 2018-03-17 01:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('slackchat', '0003_auto_20180317_0028'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='channel',
            name='image',
        ),
        migrations.RemoveField(
            model_name='channel',
            name='meta_keywords',
        ),
        migrations.AddField(
            model_name='channel',
            name='meta_image',
            field=models.URLField(blank=True, help_text='Share image URL', null=True),
        ),
        migrations.AlterField(
            model_name='channel',
            name='meta_description',
            field=models.CharField(blank=True, help_text='Page description', max_length=300, null=True),
        ),
        migrations.AlterField(
            model_name='channel',
            name='meta_title',
            field=models.CharField(blank=True, help_text='Page title', max_length=300, null=True),
        ),
    ]



# Generated by Django 2.0.2 on 2018-03-17 00:28

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('slackchat', '0002_auto_20180213_1510'),
    ]

    operations = [
        migrations.AlterField(
            model_name='channel',
            name='owner',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='slackchat.User'),
        ),
    ]



# Generated by Django 2.0.2 on 2018-03-17 02:38

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('slackchat', '0004_auto_20180317_0126'),
    ]

    operations = [
        migrations.AddField(
            model_name='chattype',
            name='publish_prefix',
            field=models.CharField(default='/slackchats/', help_text="A relative prefix that a renderer may append to a publish path when         publishing the slackchat, e.g.,         <span style='color:grey; font-weight:bold;'>/slackchats/        </span>.", max_length=300),
        ),
    ]



from rest_framework import serializers
from slackchat.models import Message

from .attachment import AttachmentSerializer
from .mixins import NoNonNullMixin
from .reaction import ReactionSerializer


class MessageSerializer(NoNonNullMixin, serializers.ModelSerializer):
    user = serializers.SerializerMethodField()
    content = serializers.SerializerMethodField()
    reactions = serializers.SerializerMethodField()
    args = serializers.SerializerMethodField()
    kwargs = serializers.SerializerMethodField()
    attachments = AttachmentSerializer(many=True, read_only=True)

    def get_user(self, obj):
        return obj.user.api_id

    def get_content(self, obj):
        if obj.channel.chat_type.render_to_html:
            return obj.html()
        else:
            return obj.get_content()

    def get_reactions(self, obj):
        reactions = []
        for reaction in obj.reactions.all():
            if not reaction.argument:
                serializer = ReactionSerializer(instance=reaction)
                reactions.append(serializer.data)

        return reactions

    def get_args(self, obj):
        args = []

        for reaction in obj.reactions.all():
            if reaction.argument:
                args.append(reaction.argument.name)

        template, match = obj.find_template_match()
        if match and template.argument_name:
            args.append(template.argument_name)

        return args

    def get_kwargs(self, obj):
        kwargs = {}

        for kwarg in obj.kwargs.all():
            kwargs[kwarg.key] = kwarg.value

        return kwargs

    class Meta:
        model = Message
        fields = (
            'timestamp',
            'user',
            'content',
            'reactions',
            'attachments',
            'args',
            'kwargs',
        )



from datetime import datetime

from django.core.exceptions import ObjectDoesNotExist
from markslack import MarkSlack
from slackchat.exceptions import MessageNotFoundError, ReactionNotFoundError
from slackchat.models import Argument, Channel, Message, Reaction, User

marker = MarkSlack()


def strptimestamp(timestamp):
    return datetime.fromtimestamp(float(timestamp))


def handle_removed(id, event):
    item = event.get('item')

    if item.get('type', None) != 'message':
        return

    item_user, created = User.objects.get_or_create(
        api_id=event.get('item_user')
    )

    user, created = User.objects.get_or_create(
        api_id=event.get('user')
    )

    try:
        channel = Channel.objects.get(
            api_id=item.get('channel')
        )
    except ObjectDoesNotExist:
        return

    try:
        message = Message.objects.get(
            timestamp=strptimestamp(item.get('ts')),
            channel=channel,
            user=item_user
        )
    except ObjectDoesNotExist:
        raise MessageNotFoundError('{} @ {} by {}'.format(
            channel,
            strptimestamp(item.get('ts')),
            item_user
        ))

    try:
        reaction = Reaction.objects.get(
            message=message,
            user=user,
            reaction=event.get('reaction'),
        )
    except ObjectDoesNotExist:
        raise ReactionNotFoundError('Can\'t find reaction.')

    reaction.delete()


def handle_added(id, event):
    item = event.get('item')

    if item.get('type', None) != 'message':
        return False

    item_user, created = User.objects.get_or_create(
        api_id=event.get('item_user')
    )

    try:
        channel = Channel.objects.get(
            api_id=item.get('channel')
        )
    except ObjectDoesNotExist:
        return

    try:
        message = Message.objects.get(
            timestamp=strptimestamp(item.get('ts')),
            channel=channel,
            user=item_user
        )
    except ObjectDoesNotExist:
        raise MessageNotFoundError('{} @ {} by {}'.format(
            channel,
            strptimestamp(item.get('ts')),
            item_user
        ))

    reaction_user, created = User.objects.get_or_create(
        api_id=event.get('user')
    )

    reaction_kwargs = {
        'timestamp': strptimestamp(event.get('event_ts')),
        'message': message,
        'reaction': event.get('reaction'),
        'user': reaction_user
    }

    try:
        argument = Argument.objects.get(character=event.get('reaction'))
        reaction_kwargs['argument'] = argument
    except ObjectDoesNotExist:
        pass

    Reaction.objects.update_or_create(**reaction_kwargs)



# Generated by Django 2.0.2 on 2018-03-17 23:28

from django.db import migrations, models
import django.utils.timezone
import slackchat.fields


class Migration(migrations.Migration):

    dependencies = [
        ('slackchat', '0005_chattype_publish_prefix'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='chattype',
            name='publish_prefix',
        ),
        migrations.AddField(
            model_name='chattype',
            name='publish_path',
            field=models.CharField(default='/slackchats/', help_text="A relative path for the slackchat type         you can append to any slackchat channel's         publish path, e.g.,         <span style='color:grey; font-weight:bold;'>/slackchats/        </span>.", max_length=300),
        ),
        migrations.AlterField(
            model_name='channel',
            name='introduction',
            field=slackchat.fields.MarkdownField(blank=True, help_text='Some introductory paragraph text (in markdown syntax).', null=True),
        ),
        migrations.AlterField(
            model_name='channel',
            name='live',
            field=models.BooleanField(default=False, help_text='Determines whether page should re-poll for new messages         while chat is live.'),
        ),
        migrations.AlterField(
            model_name='channel',
            name='publish_path',
            field=models.CharField(blank=True, help_text="A relative path you can use when         publishing the slackchat, e.g.,         <span style='color:grey; font-weight:bold;'>/2018-02-12/econ-chat/        </span>.", max_length=300, unique=True),
        ),
        migrations.AlterField(
            model_name='channel',
            name='publish_time',
            field=models.DateTimeField(default=django.utils.timezone.now, help_text='Dateline.'),
        ),
    ]



from django.db import transaction
from django.db.models.signals import post_delete, post_save
from django.dispatch import receiver

from .celery import (create_private_channel, post_webhook, update_user,
                     verify_webhook)
from .models import (Attachment, Channel, KeywordArgument, Message, Reaction,
                     User, Webhook)


@receiver(post_save, sender=Channel)
def save_channel(sender, instance, created, **kwargs):
    if created:
        create_private_channel.delay(instance.pk)


@receiver(post_save, sender=Attachment)
@receiver(post_save, sender=Message)
@receiver(post_save, sender=Reaction)
@receiver(post_save, sender=KeywordArgument)
@receiver(post_delete, sender=Attachment)
@receiver(post_delete, sender=Message)
@receiver(post_delete, sender=Reaction)
@receiver(post_delete, sender=KeywordArgument)
def notify_webhook(sender, instance, **kwargs):
    if sender == Message:
        channel_id = instance.channel.id.hex
    else:
        channel_id = instance.message.channel.id.hex
    transaction.on_commit(lambda: post_webhook.delay(channel_id))


@receiver(post_save, sender=Webhook)
def save_webhook(sender, instance, **kwargs):
    verify_webhook.delay(instance.pk)


@receiver(post_save, sender=User)
def new_user(sender, instance, created, **kwargs):
    if created:
        update_user.delay(instance.pk)



# Generated by Django 2.0.2 on 2018-03-18 15:15

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('slackchat', '0006_auto_20180317_2328'),
    ]

    operations = [
        migrations.AlterField(
            model_name='chattype',
            name='name',
            field=models.SlugField(max_length=255),
        ),
    ]



import uuid

import requests

from django.core.files.base import ContentFile
from django.core.management.base import BaseCommand
from slackchat.conf import settings
from slackchat.models import User
from slacker import Slacker
from tqdm import tqdm


class Command(BaseCommand):
    help = 'Retrieves users from Slack'

    def handle(self, *args, **options):
        SLACK = Slacker(settings.SLACK_API_TOKEN)
        slack_users = SLACK.users.list().body['members']
        for slack_user in tqdm(slack_users, desc='Users'):
            id = slack_user['id']
            profile = slack_user['profile']

            real_name = profile.get('real_name', None)
            display_name = profile.get('display_name', None)
            try:
                first_name, last_name = real_name.split(' ', 1)
            except ValueError:
                pass
            try:
                first_name, last_name = display_name.split(' ', 1)
            except ValueError:
                first_name = real_name or display_name or ''
                last_name = ''

            user, created = User.objects.update_or_create(
                api_id=id,
                defaults={
                    'first_name': first_name,
                    'last_name': last_name,
                    'email': profile.get('email', None),
                    'title': profile.get('title', 'Staff writer'),
                }
            )

            if created and profile.get('image_192', False):
                r = requests.get(profile.get('image_192'), stream=True)
                img = r.raw.read()
                user.image.save(
                    'profile-{}.jpg'.format(uuid.uuid4().hex[:10]),
                    ContentFile(img),
                    save=True
                )
            user.save()



from django.db import models


class ChatType(models.Model):
    """
    A type of slackchat.
    """

    name = models.SlugField(max_length=255)
    publish_path = models.CharField(
        default='/slackchats/',
        max_length=300,
        help_text="A relative path for the slackchat type \
        you can append to any slackchat channel's \
        publish path, e.g., \
        <span style='color:grey; font-weight:bold;'>/slackchats/\
        </span>."
    )
    render_to_html = models.BooleanField(
        default=False,
        help_text="Whether to render markdown to HTML in the serializer.")
    kwargs_in_threads = models.BooleanField(
        default=True,
        help_text="Whether users can create kwargs in threads."
    )

    def __str__(self):
        return self.name



from rest_framework import serializers
from slackchat.models import ChatType


class ChatTypeSerializer(serializers.ModelSerializer):
    class Meta:
        model = ChatType
        fields = '__all__'



from django.db import models


class Reaction(models.Model):
    """
    An emoji reaction to a message in Slack.
    """
    timestamp = models.DateTimeField(unique=True)
    message = models.ForeignKey(
        'Message', related_name='reactions', on_delete=models.CASCADE)
    reaction = models.CharField(max_length=150)
    argument = models.ForeignKey(
        'Argument', null=True, blank=True, on_delete=models.CASCADE)
    user = models.ForeignKey(
        'User', on_delete=models.CASCADE)

    def valid_action_for_role(self):
        if self.action.role:
            channel = self.message.channel
            assignments = self.message.user.get_channel_assignments(channel)

            if self.action.role in assignments:
                return True
            else:
                return False
        else:
            return True

    def __str__(self):
        return ':{0}:, reaction to "{1}"'.format(
            self.reaction,
            self.message.text
        )



from django.urls import include, path
from rest_framework import routers

from .views import Events
from .viewsets import ChannelViewset, ChatTypeViewset

router = routers.DefaultRouter()
router.register(r'channels', ChannelViewset, base_name='slackchat-channel')
router.register(
    r'chat-types', ChatTypeViewset, base_name='slackchat-chat-types')

urlpatterns = [
    path('api/', include(router.urls)),
    path('events/', Events.as_view()),
]



import uuid

import requests

from celery import shared_task
from django.core.files.base import ContentFile
from slackchat.conf import settings
from slackchat.models import User
from slacker import Slacker

SLACK = Slacker(settings.SLACK_API_TOKEN)


def get_slack_profile(user):
    """Return user profile from Slack."""
    users = SLACK.users.list().body['members']
    for slack_user in users:
        if slack_user['id'] == user.api_id:
            return slack_user['profile']
    return None


@shared_task(acks_late=True)
def update_user(pk):
    user = User.objects.get(pk=pk)
    profile = get_slack_profile(user)

    real_name = profile.get('real_name', None)
    display_name = profile.get('display_name', None)
    try:
        first_name, last_name = real_name.split(' ', 1)
    except ValueError:
        pass
    try:
        first_name, last_name = display_name.split(' ', 1)
    except ValueError:
        first_name = real_name or display_name or ''
        last_name = ''

    user.first_name = first_name
    user.last_name = last_name
    user.title = profile.get('title', 'Staff writer')
    user.email = profile.get('email', None)

    if profile.get('image_192', False):
        r = requests.get(profile.get('image_192'), stream=True)
        img = r.raw.read()
        user.image.save(
            'profile-{}.jpg'.format(uuid.uuid4().hex[:10]),
            ContentFile(img),
            save=True
        )

    user.save()



from celery import shared_task
from slackchat.conf import settings
from slackchat.models import Channel
from slackclient import SlackClient


@shared_task(acks_late=True)
def create_private_channel(pk):
    instance = Channel.objects.get(pk=pk)
    client = SlackClient(settings.SLACK_API_TOKEN)
    response = client.api_call(
        "conversations.create",
        name='slackchat-{}'.format(instance.id.hex[:10]),
        is_private=True
    )
    if response.get('ok', False):
        channel = response.get('channel')
        instance.api_id = channel.get('id')

        # Add team info
        response = client.api_call("team.info")
        if response.get('ok', False):
            team = response.get('team')
            instance.team_id = team.get('id')

        client.api_call(
            "groups.invite",
            channel=instance.api_id,
            user=instance.owner.api_id
        )
    instance.save()



from rest_framework import serializers
from slackchat.models import Attachment

from .mixins import NoNonNullMixin


class AttachmentSerializer(NoNonNullMixin, serializers.ModelSerializer):

    class Meta:
        model = Attachment
        fields = (
            'title',
            'title_link',
            'text',
            'service_name',
            'service_icon',
            'service_url',
            'image_url',
            'image_width',
            'image_height',
            'video_html',
            'video_html_width',
            'video_html_height',
            'thumb_url',
            'thumb_width',
            'thumb_height',
        )



# flake8: noqa
from .messages import handle as handle_message
from .messages import handle_removed as handle_message_removed
from .reactions import handle_added as handle_reaction_added
from .reactions import handle_removed as handle_reaction_removed



from rest_framework import serializers
from slackchat.models import User


class UserSerializer(serializers.ModelSerializer):
    class Meta:
        model = User
        fields = (
            'first_name',
            'last_name',
            'image',
            'title'
        )



# Generated by Django 2.0.2 on 2018-02-10 18:05

import uuid

import django.db.models.deletion
import django.utils.timezone
import slackchat.conf
import slackchat.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    replaces = [('slackchat', '0001_initial'), ('slackchat', '0002_auto_20180207_0014'), ('slackchat', '0003_auto_20180207_1326'), ('slackchat', '0004_auto_20180207_1332'), ('slackchat', '0005_attachment'), ('slackchat', '0006_auto_20180210_0422'), ('slackchat', '0007_auto_20180210_0428'), ('slackchat', '0008_auto_20180210_1230'), ('slackchat', '0009_auto_20180210_1403'), ('slackchat', '0010_auto_20180210_1726')]

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Argument',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.SlugField(help_text='The name of the argument that will be attached to a          message.', max_length=255)),
                ('character', models.CharField(max_length=100)),
            ],
        ),
        migrations.CreateModel(
            name='Channel',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('api_id', models.SlugField(blank=True, editable=False, max_length=10, null=True)),
                ('owner', models.CharField(choices=[('U1SPM8DC0', 'aabril@politico.com'), ('U28R24K1C', 'aadragna@politico.com'), ('U2RMM881Y', 'abehsudi@politico.com'), ('U301T5ATU', 'acancryn@politico.com'), ('U4U8L8JUB', 'ahurst@politico.com'), ('U2ZJSDJUD', 'agreilingkeane@politico.com'), ('U2K6K7S4W', 'ahanna@politico.com'), ('U3TC14MEK', 'ahoffman@politico.com'), ('U0AN43XCH', 'aisenstadt@politico.com'), ('U2QUJGMNJ', 'ajakabcin@politico.com'), ('U081UE9C0', 'akarni@politico.com'), ('U6KS7CBV3', 'alacy@politico.com'), ('U37NZ0849', 'avelickovich@politico.com'), ('U0BTDEKLH', 'aguillen@politico.com'), ('U439APEE7', 'aphelps@politico.com'), ('U90RT4Q2G', 'alorenzo@politico.com'), ('U1VSTAGJV', 'athorne@politico.com'), ('U0BJVM7E0', 'agoodwin@politico.com'), ('U1MF077LK', 'astubbs@politico.com'), ('U0L1545UY', 'apalmer@politico.com'), ('U0BTA2GKX', 'arestuccia@politico.com'), ('U0FHGMZ4K', 'asnider@politico.com'), ('U2ZMG3S2F', 'aweaver@politico.com'), ('U8X69UACU', 'ayannaalexander@politico.com'), ('U6W564BCM', 'bbecker@politico.com'), ('U0NS8TMAB', 'bdayspring@politico.com'), ('U464U0N78', 'blefebvre@politico.com'), ('U2ANMR2US', 'bweyl@politico.com'), ('U0AP1C5NE', 'bwhite@politico.com'), ('U2BA099NV', 'bgriffiths@politico.com'), ('U113DBRBK', 'bgurciullo@politico.com'), ('U2RLNMEUR', 'bhillman@politico.com'), ('U1F3P2X2Q', 'bpadro@politico.com'), ('U0CHY31KR', 'bduryea@politico.com'), ('U0C2LLUH2', 'bkuchman@politico.com'), ('U0ALHHQBU', 'bking@politico.com'), ('U04CVTYKA', 'bhounshell@politico.com'), ('U4JCNVDC7', 'bmacarthur@politico.com'), ('U198YQCGY', 'beverett@politico.com'), ('U0A4SEGD9', 'bpeterson@politico.com'), ('U2N69NF2L', 'bqatipi@politico.com'), ('U0944177F', 'behley@politico.com'), ('U0ANB6GQ2', 'bmulcahy@politico.com'), ('U301PTEEQ', 'britchie@politico.com'), ('U304P9CJH', 'bwermund@politico.com'), ('U0X91BK5J', 'coconnell@politico.com'), ('U2ZRESPDG', 'coprysko@politico.com'), ('U1SQEGZ24', 'ckelly@politico.com'), ('U2YP3RKPW', 'cbrown@politico.com'), ('U15B2SW6P', 'cbaute@politico.com'), ('U2YUKJ84U', 'cbudoff@politico.com'), ('U4HGYBCR1', 'cbrownell@politico.com'), ('U0AQY8545', 'cemma@politico.com'), ('U5N08DWD8', 'cfoxwell@politico.com'), ('U0607KKA4', 'cmahtesian@politico.com'), ('U6Z6947B2', 'ckarna@politico.com'), ('U0AQFE4KS', 'cbenson@politico.com'), ('U178FC98D', 'clima@politico.com'), ('U1TH6E94J', 'cobrien@politico.com'), ('U0AQVKWG5', 'cwilhelm@politico.com'), ('U2M6CK0DR', 'cbennett@politico.com'), ('U8RRRTVJB', 'cprice@politico.com'), ('U09EX4ZLG', 'chowie@politico.com'), ('U0LP8J2P4', 'cschneier@politico.com'), ('U38NANURX', 'dbishop@politico.com'), ('U06JPATP1', 'dstrauss@politico.com'), ('U0AQZ7448', 'dvinik@politico.com'), ('U1EUY2ZQ8', 'dspinelli@politico.com'), ('U09HL8GSK', 'dcohen@politico.com'), ('U084VCHSS', 'ddabruzzo@politico.com'), ('U1SS3J952', 'ddiamond@politico.com'), ('U0BTG4U57', 'ddixon@politico.com'), ('U2YMUBL80', 'dducassi@politico.com'), ('U5TAQBHT7', 'daurelius@politico.com'), ('U0NAMU8S3', 'bbender@politico.com'), ('U09CDS23W', 'dkihara@politico.com'), ('U07V2P7DW', 'dlippman@politico.com'), ('U1EV3DT9Q', 'dharrell@politico.com'), ('U3TNJ51C2', 'dpalmer@politico.com'), ('U7167DFAL', 'drobertson@politico.com'), ('U0ANFSDF0', 'dsamuelsohn@politico.com'), ('U09HDQPEJ', 'eschneider@politico.com'), ('U8X3PNPGT', 'ecastillo@politico.com'), ('U1SPGL1RS', 'edobler@politico.com'), ('U0AQN8J20', 'eengleman@politico.com'), ('U6H65M171', 'eholden@politico.com'), ('U0AP412SC', 'ceisenberg@politico.com'), ('U36UAFTF0', 'ejohnson@politico.com'), ('U09UDM00Y', 'eralph@politico.com'), ('U69LA48BA', 'egoldberg@politico.com'), ('U4R16RK53', 'estephenson@politico.com'), ('U8R1U1BFG', 'eokun@politico.com'), ('U2MT2TJJD', 'egeller@politico.com'), ('U0BTAG7FA', 'ewolff@politico.com'), ('U51ACPMHQ', 'eriley@politico.com'), ('U0LKACSQM', 'eaulov@politico.com'), ('U0BTCMKD3', 'eschor@politico.com'), ('U07TUTP16', 'gdebenedetti@politico.com'), ('U4M1RFUHJ', 'gbrotman@politico.com'), ('U720TJXA9', 'gglatsky@politico.com'), ('U4HH5Q6H1', 'ggoodman@politico.com'), ('U07V86U02', 'hjackson@politico.com'), ('U1B64GTFT', 'hcaygle@politico.com'), ('U2G6R9VFD', 'ikullgren@politico.com'), ('U2G8EP16C', 'isaac@politico.com'), ('U2LQ299RV', 'jheinz@politico.com'), ('U130UQ0EQ', 'jsmith@politico.com'), ('U1819GV42', 'jsherman@politico.com'), ('U1GTY36RG', 'jamster@politico.com'), ('U0AQ1PHGX', 'jmillman@politico.com'), ('U0LRG5D9B', 'jbresnahan@politico.com'), ('U1FHQQRFX', 'jcuellar@politico.com'), ('U5PGG84PP', 'jlin@politico.com'), ('U30JE92G5', 'jblount@politico.com'), ('U0AQXQF0X', 'jhaberkorn@politico.com'), ('U7E5R7A2G', 'jklimas@politico.com'), ('U56DEADT9', 'jmichaud@politico.com'), ('U4HMKDDNZ', 'jmiller@politico.com'), ('U0ANY3D43', 'jkenen@politico.com'), ('U1DPMRWHL', 'jlauinger@politico.com'), ('U4XV32XKR', 'jmcclure@politico.com'), ('U5GUJNRPX', 'jhalling@politico.com'), ('U0BH0V25R', 'jgerstein@politico.com'), ('U5RGFNUTH', 'jmeyer@politico.com'), ('U50NR064T', 'jplesniak@politico.com'), ('U0AQVA6ES', 'jscholtes@politico.com'), ('U6V6HGR2P', 'jschwartz@politico.com'), ('U8PMG51NU', 'judahtaylor@politico.com'), ('U2LR2CHR9', 'jzbenson@politico.com'), ('U04RJGPT5', 'kareywutkowski@gmail.com'), ('U13JDSSKA', 'kmckibben@politico.com'), ('U2ZMJ6RRB', 'klandergan@politico.com'), ('U497JU6KF', 'kmoukhina@politico.com'), ('U0BJZGKDW', 'kbarnard@politico.com'), ('U1SPSSKEZ', 'kburton@politico.com'), ('U07UQH5NV', 'kcheney@politico.com'), ('U0AN64CLU', 'keast@politico.com'), ('U09GTA39V', 'krobillard@politico.com'), ('U0AMS55RS', 'kfossett@politico.com'), ('U0APZM2CE', 'khefling@politico.com'), ('U8Q7R58RF', 'kmakle@politico.com'), ('U1SPJ7K9S', 'kmiller@politico.com'), ('U4GB9N0JZ', 'kpudwill@politico.com'), ('U2LSW1971', 'krector@politico.com'), ('U75FQ4PQA', 'krice@politico.com'), ('U5ZKPKYRE', 'ktamborrino@politico.com'), ('U0PGV8WQ7', 'kodonnell@politico.com'), ('U3D664FML', 'kwyatt@politico.com'), ('U2ZLYL8JH', 'ldezenski@politico.com'), ('U2M81FX5F', 'lgardner@politico.com'), ('U4XSAAAH3', 'lmihalik@politico.com'), ('U2MDKHCDV', 'lmertz@politico.com'), ('U2QRVQ6P5', 'lli@politico.com'), ('U3GPCLXPF', 'lmonaghan@politico.com'), ('U3BT4V483', 'lmuntzing@politico.com'), ('U2M7Z20MT', 'lwoellert@politico.com'), ('U09E71A85', 'lnelson@politico.com'), ('U6Z0YLEA0', 'lsanchez@politico.com'), ('U0CAP3QRW', 'mseverns@politico.com'), ('U0A051TCJ', 'mslattery@politico.com'), ('U30E4RVK8', 'mmatishak@politico.com'), ('U0AQQBENS', 'mdaily@politico.com'), ('U0QNU6989', 'mnussbaum@politico.com'), ('U0C2M90R2', 'mbloom@politico.com'), ('U770N1KKQ', 'mcalderone@politico.com'), ('U4BSQ51T7', 'mcassella@politico.com'), ('U0AN6DFU2', 'mcrowley@politico.com'), ('U2LTDP1BK', 'michellez@politico.com'), ('U0800UDKJ', 'mzapler@politico.com'), ('U1HGCCF71', 'mirwin@politico.com'), ('U0HAATT9N', 'mschuler@politico.com'), ('U0AMXDNKW', 'mkady@politico.com'), ('U1SPSEDK9', 'mlee@politico.com'), ('U2RM8UHL6', 'mleonor@politico.com'), ('U2RM777JL', 'mlevine@politico.com'), ('U09R4G4EM', 'mreynolds@politico.com'), ('U8R2YCN9X', 'mrodriguez@politico.com'), ('U0NT2MQE9', 'mswiatkowski@politico.com'), ('U54A64Z4K', 'nfreiberg@politico.com'), ('U0AQVA1DY', 'njuliano@politico.com'), ('U1W37ACQ4', 'nnarea@politico.com'), ('U0E7MP405', 'nmccaskill@politico.com'), ('U0AN62LR0', 'ntoosi@politico.com'), ('U8M5NSEUC', 'odraughon@politico.com'), ('U8RR8BFTR', 'pdsilva@politico.com'), ('U0AQSD4HW', 'pcanellos@politico.com'), ('U5MD6AFFB', 'pjoshi@politico.com'), ('U0PQAF7K3', 'ptemple-west@politico.com'), ('U3B0ZV8KE', 'pvolpe@politico.com'), ('U0AQ90A4F', 'rpradhan@politico.com'), ('U30Q7KXQW', 'rarrieta-kenna@politico.com'), ('U1X48QNSE', 'rshamon@politico.com'), ('U0BK1C1QU', 'rbade@politico.com'), ('U49LANBQB', 'rflores@politico.com'), ('U6CCB91AM', 'rlemmerman@politico.com'), ('U1M105C72', 'rmorin@politico.com'), ('U6RK9984U', 'rrobinson@politico.com'), ('U0BJVV3HR', 'rturner@politico.com'), ('U5GTA0H6Y', 'sbarnes@politico.com'), ('U8R036EP6', 'samsmith@politico.com'), ('U5SFBECBS', 'sfrostenson@politico.com'), ('U8DFBLWPL', 'sbeynon@politico.com'), ('U2Z6AV6SJ', 'sbustos@politico.com'), ('U0E8GMTFU', 'sbland@politico.com'), ('U4HK8H4CB', 'seppler@politico.com'), ('U198EQDG9', 'skim@politico.com'), ('U0AN15WJ0', 'sheuser@politico.com'), ('U4BC16USV', 'slackadmin@politico.com'), ('U6S00T63C', 'smajid@politico.com'), ('U09R2KN7R', 'msobocinski@politico.com'), ('U2LSL5K1N', 'swillis@politico.com'), ('U38EZ7FEW', 'srafferty@politico.com'), ('U8QDKNZ4H', 'srahmani@politico.com'), ('U8YU59HB8', 'srodriguez@politico.com'), ('U09HENNDB', 'sshepard@politico.com'), ('U3LRDATQA', 'sreddy@politico.com'), ('U0BJXRA95', 'ssagar@politico.com'), ('U2U3NBQLT', 'tsnyder@politico.com'), ('U6XTN00QH', 'tthomas@politico.com'), ('U28LCU7MF', 'tgee@politico.com'), ('U0V3PC43F', 'tdoherty@politico.com'), ('U2RMS9482', 'thesson@politico.com'), ('U09E8G563', 'tmeyer@politico.com'), ('U6SBQQ86R', 'tlindeman@politico.com'), ('U2RL0NL31', 'tsperling@politico.com'), ('U30NKLTSS', 'tstarks@politico.com'), ('U2LSQK9TP', 'tweyant@politico.com'), ('U1GT60ZMF', 'tpager@politico.com'), ('U5U0DHNB1', 'tfisher@politico.com'), ('U2ZJ3P8AH', 'vguida@politico.com'), ('U2LT1TYVC', 'whouseknecht@politico.com'), ('U5V8MKTV5', 'jyuan@politico.com'), ('U0PQ4TK5E', 'zwarmbrodt@politico.com'), ('U0Y57RBRB', 'zmontellaro@politico.com'), ('U0USU8V0B', 'zstanton@politico.com')], max_length=200)),
                ('title', models.CharField(blank=True, max_length=150, null=True)),
                ('image', models.ImageField(blank=True, help_text='An image to feature on the rendered Slackchat page.', null=True)),
                ('introduction', models.TextField(blank=True, help_text='Some introductory paragraph text in markdown syntax.', null=True)),
                ('meta_title', models.CharField(blank=True, help_text='Title for page meta data.', max_length=300, null=True)),
                ('meta_description', models.CharField(blank=True, help_text='Description for page meta data.', max_length=300, null=True)),
                ('meta_keywords', models.CharField(blank=True, help_text='Keywords for page meta data.', max_length=300, null=True)),
                ('publish_path', models.CharField(blank=True, help_text='A relative path that a renderer may use when         publishing the slackchat.', max_length=300, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='ChatType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('render_to_html', models.BooleanField(default=False, help_text='Whether to render markdown to HTML in the serializer.')),
                ('kwargs_in_threads', models.BooleanField(default=True, help_text='Whether users can create kwargs in threads.')),
            ],
        ),
        migrations.CreateModel(
            name='CustomContentTemplate',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('argument_name', models.SlugField(blank=True, help_text="Add an argument to the message if search_string         matches against a message's content.", max_length=255, null=True)),
                ('search_string', models.CharField(help_text='A regex search string with capture groups.', max_length=255)),
                ('content_template', models.TextField(help_text='A Python format string whose args are the capture groups         matched by the search_string.')),
                ('chat_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='slackchat.ChatType')),
            ],
        ),
        migrations.CreateModel(
            name='KeywordArgument',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('timestamp', models.DateTimeField(unique=True)),
                ('key', models.SlugField(max_length=30)),
                ('value', models.TextField()),
            ],
        ),
        migrations.CreateModel(
            name='Message',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('timestamp', models.DateTimeField(unique=True)),
                ('text', models.TextField()),
                ('channel', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='messages', to='slackchat.Channel')),
            ],
        ),
        migrations.CreateModel(
            name='Reaction',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('timestamp', models.DateTimeField(unique=True)),
                ('reaction', models.CharField(max_length=150)),
                ('argument', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='slackchat.Argument')),
                ('message', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='reactions', to='slackchat.Message')),
            ],
        ),
        migrations.CreateModel(
            name='User',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('api_id', models.CharField(max_length=50)),
                ('first_name', models.CharField(blank=True, max_length=100, null=True)),
                ('last_name', models.CharField(blank=True, max_length=200, null=True)),
                ('email', models.EmailField(blank=True, max_length=254, null=True)),
                ('image', models.ImageField(upload_to=slackchat.conf.default_user_image_upload_to)),
                ('title', models.CharField(max_length=255)),
            ],
        ),
        migrations.CreateModel(
            name='Webhook',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('endpoint', models.URLField(unique=True)),
                ('verified', models.BooleanField(default=False)),
            ],
        ),
        migrations.AddField(
            model_name='reaction',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='slackchat.User'),
        ),
        migrations.AddField(
            model_name='message',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='messages', to='slackchat.User'),
        ),
        migrations.AddField(
            model_name='keywordargument',
            name='message',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='kwargs', to='slackchat.Message'),
        ),
        migrations.AddField(
            model_name='keywordargument',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='slackchat.User'),
        ),
        migrations.AddField(
            model_name='channel',
            name='chat_type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='slackchat.ChatType'),
        ),
        migrations.AddField(
            model_name='argument',
            name='chat_type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='slackchat.ChatType'),
        ),
        migrations.AlterField(
            model_name='channel',
            name='publish_path',
            field=models.CharField(blank=True, help_text="A relative path that a renderer may use when         publishing the slackchat, e.g.,         <span style='color:grey; font-weight:bold;'>/2018-02-12/econ-chat/        </span>.", max_length=300, unique=True),
        ),
        migrations.AddField(
            model_name='channel',
            name='live',
            field=models.BooleanField(default=False, help_text='May be used by renderer to determine        if it should repoll for messages while chat is live.'),
        ),
        migrations.AddField(
            model_name='channel',
            name='publish_time',
            field=models.DateTimeField(default=django.utils.timezone.now, help_text='May be used by renderer to         create dateline.'),
        ),
        migrations.CreateModel(
            name='Attachment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(blank=True, max_length=300, null=True)),
                ('title_link', models.URLField(blank=True, null=True)),
                ('text', models.TextField(blank=True, null=True)),
                ('image_url', models.URLField(blank=True, null=True)),
                ('image_width', models.PositiveSmallIntegerField(blank=True, null=True)),
                ('image_height', models.PositiveSmallIntegerField(blank=True, null=True)),
                ('service_name', models.CharField(blank=True, max_length=300, null=True)),
                ('service_icon', models.URLField(blank=True, null=True)),
                ('message', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='attachments', to='slackchat.Message')),
                ('service_url', models.URLField(blank=True, null=True)),
                ('thumb_height', models.PositiveSmallIntegerField(blank=True, null=True)),
                ('thumb_url', models.URLField(blank=True, null=True)),
                ('thumb_width', models.PositiveSmallIntegerField(blank=True, null=True)),
                ('video_html', models.TextField(blank=True, null=True)),
                ('video_html_height', models.PositiveSmallIntegerField(blank=True, null=True)),
                ('video_html_width', models.PositiveSmallIntegerField(blank=True, null=True)),
            ],
        ),
        migrations.AlterUniqueTogether(
            name='attachment',
            unique_together={('message', 'title_link', 'image_url')},
        ),
        migrations.AlterField(
            model_name='channel',
            name='introduction',
            field=slackchat.fields.MarkdownField(blank=True, help_text='Some introductory paragraph text in markdown syntax.', null=True),
        ),
        migrations.AlterField(
            model_name='channel',
            name='owner',
            field=models.CharField(choices=[('U1SPM8DC0', 'aabril@politico.com'), ('U28R24K1C', 'aadragna@politico.com'), ('U2RMM881Y', 'abehsudi@politico.com'), ('U301T5ATU', 'acancryn@politico.com'), ('U4U8L8JUB', 'ahurst@politico.com'), ('U2ZJSDJUD', 'agreilingkeane@politico.com'), ('U2K6K7S4W', 'ahanna@politico.com'), ('U3TC14MEK', 'ahoffman@politico.com'), ('U0AN43XCH', 'aisenstadt@politico.com'), ('U2QUJGMNJ', 'ajakabcin@politico.com'), ('U081UE9C0', 'akarni@politico.com'), ('U6KS7CBV3', 'alacy@politico.com'), ('U37NZ0849', 'avelickovich@politico.com'), ('U0BTDEKLH', 'aguillen@politico.com'), ('U439APEE7', 'aphelps@politico.com'), ('U90RT4Q2G', 'alorenzo@politico.com'), ('U1VSTAGJV', 'athorne@politico.com'), ('U0BJVM7E0', 'agoodwin@politico.com'), ('U1MF077LK', 'astubbs@politico.com'), ('U0L1545UY', 'apalmer@politico.com'), ('U0BTA2GKX', 'arestuccia@politico.com'), ('U0FHGMZ4K', 'asnider@politico.com'), ('U2ZMG3S2F', 'aweaver@politico.com'), ('U8X69UACU', 'ayannaalexander@politico.com'), ('U6W564BCM', 'bbecker@politico.com'), ('U0NS8TMAB', 'bdayspring@politico.com'), ('U464U0N78', 'blefebvre@politico.com'), ('U2ANMR2US', 'bweyl@politico.com'), ('U0AP1C5NE', 'bwhite@politico.com'), ('U2BA099NV', 'bgriffiths@politico.com'), ('U113DBRBK', 'bgurciullo@politico.com'), ('U2RLNMEUR', 'bhillman@politico.com'), ('U1F3P2X2Q', 'bpadro@politico.com'), ('U0CHY31KR', 'bduryea@politico.com'), ('U0C2LLUH2', 'bkuchman@politico.com'), ('U0ALHHQBU', 'bking@politico.com'), ('U04CVTYKA', 'bhounshell@politico.com'), ('U4JCNVDC7', 'bmacarthur@politico.com'), ('U198YQCGY', 'beverett@politico.com'), ('U0A4SEGD9', 'bpeterson@politico.com'), ('U2N69NF2L', 'bqatipi@politico.com'), ('U0944177F', 'behley@politico.com'), ('U0ANB6GQ2', 'bmulcahy@politico.com'), ('U301PTEEQ', 'britchie@politico.com'), ('U304P9CJH', 'bwermund@politico.com'), ('U0X91BK5J', 'coconnell@politico.com'), ('U2ZRESPDG', 'coprysko@politico.com'), ('U1SQEGZ24', 'ckelly@politico.com'), ('U2YP3RKPW', 'cbrown@politico.com'), ('U15B2SW6P', 'cbaute@politico.com'), ('U2YUKJ84U', 'cbudoff@politico.com'), ('U4HGYBCR1', 'cbrownell@politico.com'), ('U0AQY8545', 'cemma@politico.com'), ('U5N08DWD8', 'cfoxwell@politico.com'), ('U0607KKA4', 'cmahtesian@politico.com'), ('U6Z6947B2', 'ckarna@politico.com'), ('U0AQFE4KS', 'cbenson@politico.com'), ('U178FC98D', 'clima@politico.com'), ('U1TH6E94J', 'cobrien@politico.com'), ('U0AQVKWG5', 'cwilhelm@politico.com'), ('U2M6CK0DR', 'cbennett@politico.com'), ('U8RRRTVJB', 'cprice@politico.com'), ('U09EX4ZLG', 'chowie@politico.com'), ('U0LP8J2P4', 'cschneier@politico.com'), ('U38NANURX', 'dbishop@politico.com'), ('U06JPATP1', 'dstrauss@politico.com'), ('U0AQZ7448', 'dvinik@politico.com'), ('U1EUY2ZQ8', 'dspinelli@politico.com'), ('U09HL8GSK', 'dcohen@politico.com'), ('U084VCHSS', 'ddabruzzo@politico.com'), ('U1SS3J952', 'ddiamond@politico.com'), ('U0BTG4U57', 'ddixon@politico.com'), ('U2YMUBL80', 'dducassi@politico.com'), ('U5TAQBHT7', 'daurelius@politico.com'), ('U0NAMU8S3', 'bbender@politico.com'), ('U09CDS23W', 'dkihara@politico.com'), ('U07V2P7DW', 'dlippman@politico.com'), ('U1EV3DT9Q', 'dharrell@politico.com'), ('U3TNJ51C2', 'dpalmer@politico.com'), ('U7167DFAL', 'drobertson@politico.com'), ('U0ANFSDF0', 'dsamuelsohn@politico.com'), ('U09HDQPEJ', 'eschneider@politico.com'), ('U8X3PNPGT', 'ecastillo@politico.com'), ('U1SPGL1RS', 'edobler@politico.com'), ('U0AQN8J20', 'eengleman@politico.com'), ('U6H65M171', 'eholden@politico.com'), ('U0AP412SC', 'ceisenberg@politico.com'), ('U36UAFTF0', 'ejohnson@politico.com'), ('U09UDM00Y', 'eralph@politico.com'), ('U69LA48BA', 'egoldberg@politico.com'), ('U4R16RK53', 'estephenson@politico.com'), ('U8R1U1BFG', 'eokun@politico.com'), ('U2MT2TJJD', 'egeller@politico.com'), ('U0BTAG7FA', 'ewolff@politico.com'), ('U51ACPMHQ', 'eriley@politico.com'), ('U0LKACSQM', 'eaulov@politico.com'), ('U0BTCMKD3', 'eschor@politico.com'), ('U07TUTP16', 'gdebenedetti@politico.com'), ('U4M1RFUHJ', 'gbrotman@politico.com'), ('U720TJXA9', 'gglatsky@politico.com'), ('U4HH5Q6H1', 'ggoodman@politico.com'), ('U07V86U02', 'hjackson@politico.com'), ('U1B64GTFT', 'hcaygle@politico.com'), ('U2G6R9VFD', 'ikullgren@politico.com'), ('U2G8EP16C', 'isaac@politico.com'), ('U2LQ299RV', 'jheinz@politico.com'), ('U130UQ0EQ', 'jsmith@politico.com'), ('U1819GV42', 'jsherman@politico.com'), ('U1GTY36RG', 'jamster@politico.com'), ('U0AQ1PHGX', 'jmillman@politico.com'), ('U0LRG5D9B', 'jbresnahan@politico.com'), ('U1FHQQRFX', 'jcuellar@politico.com'), ('U5PGG84PP', 'jlin@politico.com'), ('U30JE92G5', 'jblount@politico.com'), ('U0AQXQF0X', 'jhaberkorn@politico.com'), ('U7E5R7A2G', 'jklimas@politico.com'), ('U56DEADT9', 'jmichaud@politico.com'), ('U4HMKDDNZ', 'jmiller@politico.com'), ('U0ANY3D43', 'jkenen@politico.com'), ('U1DPMRWHL', 'jlauinger@politico.com'), ('U4XV32XKR', 'jmcclure@politico.com'), ('U5GUJNRPX', 'jhalling@politico.com'), ('U0BH0V25R', 'jgerstein@politico.com'), ('U5RGFNUTH', 'jmeyer@politico.com'), ('U50NR064T', 'jplesniak@politico.com'), ('U0AQVA6ES', 'jscholtes@politico.com'), ('U6V6HGR2P', 'jschwartz@politico.com'), ('U8PMG51NU', 'judahtaylor@politico.com'), ('U2LR2CHR9', 'jzbenson@politico.com'), ('U04RJGPT5', 'kareywutkowski@gmail.com'), ('U13JDSSKA', 'kmckibben@politico.com'), ('U2ZMJ6RRB', 'klandergan@politico.com'), ('U497JU6KF', 'kmoukhina@politico.com'), ('U0BJZGKDW', 'kbarnard@politico.com'), ('U1SPSSKEZ', 'kburton@politico.com'), ('U07UQH5NV', 'kcheney@politico.com'), ('U0AN64CLU', 'keast@politico.com'), ('U09GTA39V', 'krobillard@politico.com'), ('U0AMS55RS', 'kfossett@politico.com'), ('U0APZM2CE', 'khefling@politico.com'), ('U8Q7R58RF', 'kmakle@politico.com'), ('U1SPJ7K9S', 'kmiller@politico.com'), ('U4GB9N0JZ', 'kpudwill@politico.com'), ('U2LSW1971', 'krector@politico.com'), ('U75FQ4PQA', 'krice@politico.com'), ('U5ZKPKYRE', 'ktamborrino@politico.com'), ('U0PGV8WQ7', 'kodonnell@politico.com'), ('U3D664FML', 'kwyatt@politico.com'), ('U2ZLYL8JH', 'ldezenski@politico.com'), ('U2M81FX5F', 'lgardner@politico.com'), ('U4XSAAAH3', 'lmihalik@politico.com'), ('U2MDKHCDV', 'lmertz@politico.com'), ('U2QRVQ6P5', 'lli@politico.com'), ('U3GPCLXPF', 'lmonaghan@politico.com'), ('U3BT4V483', 'lmuntzing@politico.com'), ('U2M7Z20MT', 'lwoellert@politico.com'), ('U09E71A85', 'lnelson@politico.com'), ('U6Z0YLEA0', 'lsanchez@politico.com'), ('U0CAP3QRW', 'mseverns@politico.com'), ('U0A051TCJ', 'mslattery@politico.com'), ('U30E4RVK8', 'mmatishak@politico.com'), ('U0AQQBENS', 'mdaily@politico.com'), ('U0QNU6989', 'mnussbaum@politico.com'), ('U0C2M90R2', 'mbloom@politico.com'), ('U770N1KKQ', 'mcalderone@politico.com'), ('U4BSQ51T7', 'mcassella@politico.com'), ('U0AN6DFU2', 'mcrowley@politico.com'), ('U2LTDP1BK', 'michellez@politico.com'), ('U0800UDKJ', 'mzapler@politico.com'), ('U1HGCCF71', 'mirwin@politico.com'), ('U0HAATT9N', 'mschuler@politico.com'), ('U0AMXDNKW', 'mkady@politico.com'), ('U1SPSEDK9', 'mlee@politico.com'), ('U2RM8UHL6', 'mleonor@politico.com'), ('U2RM777JL', 'mlevine@politico.com'), ('U09R4G4EM', 'mreynolds@politico.com'), ('U8R2YCN9X', 'mrodriguez@politico.com'), ('U0NT2MQE9', 'mswiatkowski@politico.com'), ('U54A64Z4K', 'nfreiberg@politico.com'), ('U0AQVA1DY', 'njuliano@politico.com'), ('U1W37ACQ4', 'nnarea@politico.com'), ('U0E7MP405', 'nmccaskill@politico.com'), ('U0AN62LR0', 'ntoosi@politico.com'), ('U8M5NSEUC', 'odraughon@politico.com'), ('U8RR8BFTR', 'pdsilva@politico.com'), ('U0AQSD4HW', 'pcanellos@politico.com'), ('U5MD6AFFB', 'pjoshi@politico.com'), ('U0PQAF7K3', 'ptemple-west@politico.com'), ('U3B0ZV8KE', 'pvolpe@politico.com'), ('U0AQ90A4F', 'rpradhan@politico.com'), ('U30Q7KXQW', 'rarrieta-kenna@politico.com'), ('U1X48QNSE', 'rshamon@politico.com'), ('U0BK1C1QU', 'rbade@politico.com'), ('U49LANBQB', 'rflores@politico.com'), ('U6CCB91AM', 'rlemmerman@politico.com'), ('U1M105C72', 'rmorin@politico.com'), ('U6RK9984U', 'rrobinson@politico.com'), ('U0BJVV3HR', 'rturner@politico.com'), ('U5GTA0H6Y', 'sbarnes@politico.com'), ('U8R036EP6', 'samsmith@politico.com'), ('U5SFBECBS', 'sfrostenson@politico.com'), ('U8DFBLWPL', 'sbeynon@politico.com'), ('U2Z6AV6SJ', 'sbustos@politico.com'), ('U0E8GMTFU', 'sbland@politico.com'), ('U4HK8H4CB', 'seppler@politico.com'), ('U0AN15WJ0', 'sheuser@politico.com'), ('U4BC16USV', 'slackadmin@politico.com'), ('U6S00T63C', 'smajid@politico.com'), ('U09R2KN7R', 'msobocinski@politico.com'), ('U2LSL5K1N', 'swillis@politico.com'), ('U38EZ7FEW', 'srafferty@politico.com'), ('U8QDKNZ4H', 'srahmani@politico.com'), ('U8YU59HB8', 'srodriguez@politico.com'), ('U09HENNDB', 'sshepard@politico.com'), ('U3LRDATQA', 'sreddy@politico.com'), ('U0BJXRA95', 'ssagar@politico.com'), ('U2U3NBQLT', 'tsnyder@politico.com'), ('U6XTN00QH', 'tthomas@politico.com'), ('U28LCU7MF', 'tgee@politico.com'), ('U0V3PC43F', 'tdoherty@politico.com'), ('U2RMS9482', 'thesson@politico.com'), ('U09E8G563', 'tmeyer@politico.com'), ('U6SBQQ86R', 'tlindeman@politico.com'), ('U2RL0NL31', 'tsperling@politico.com'), ('U30NKLTSS', 'tstarks@politico.com'), ('U2LSQK9TP', 'tweyant@politico.com'), ('U1GT60ZMF', 'tpager@politico.com'), ('U5U0DHNB1', 'tfisher@politico.com'), ('U2ZJ3P8AH', 'vguida@politico.com'), ('U2LT1TYVC', 'whouseknecht@politico.com'), ('U5V8MKTV5', 'jyuan@politico.com'), ('U0PQ4TK5E', 'zwarmbrodt@politico.com'), ('U0Y57RBRB', 'zmontellaro@politico.com'), ('U0USU8V0B', 'zstanton@politico.com')], max_length=200),
        ),
    ]



from datetime import datetime

from django.core.exceptions import ObjectDoesNotExist
from markslack import MarkSlack

from slackchat.conf import settings
from slackchat.exceptions import MessageNotFoundError, UserNotFoundError
from slackchat.models import Channel, KeywordArgument, Message, User

from .attachments import handle as handle_attachment

ignored_subtypes = [
    'group_join',
    'file_share',
    'group_archive',
]


def strptimestamp(timestamp):
    return datetime.fromtimestamp(float(timestamp))


def handle_removed(id, event):
    try:
        channel = Channel.objects.get(
            api_id=event.get('channel')
        )
    except ObjectDoesNotExist:
        return
    msg = {
        'user': event.get('previous_message').get('user'),
        'ts': event.get('previous_message').get('ts'),
        'text': event.get('message', {}).get('text', None)
    }
    user, created = User.objects.get_or_create(
        api_id=msg.get('user')
    )

    # If thread
    if event.get('previous_message', {}).get('thread_ts'):
        if not channel.chat_type.kwargs_in_threads:
            return

        thread = event.get('previous_message')

        try:
            original_message = Message.objects.get(
                timestamp=strptimestamp(thread.get('thread_ts'))
            )
        except ObjectDoesNotExist:
            raise MessageNotFoundError(
                '{}'.format(strptimestamp(thread.get('thread_ts')))
            )

        try:
            user = User.objects.get(api_id=thread.get('user'))
        except ObjectDoesNotExist:
            raise UserNotFoundError(thread.get('user'))

        try:
            kwarg = KeywordArgument.objects.get(
                timestamp=strptimestamp(thread.get('ts')),
                message=original_message,
                user=user
            )
        except ObjectDoesNotExist:
            return

        kwarg.delete()

    else:
        Message.objects.get(
            channel=channel,
            timestamp=strptimestamp(msg.get('ts')),
            user=user,
        ).delete()


def handle(id, event):
    user_templates = {
        user.api_id: settings.MARKSLACK_USER_TEMPLATE(user)
        for user in User.objects.all()
    }

    marker = MarkSlack(
        user_templates=user_templates,
        link_templates=settings.MARKSLACK_LINK_TEMPLATES,
        image_template=settings.MARKSLACK_IMAGE_TEMPLATE
    )

    try:
        channel = Channel.objects.get(
            api_id=event.get('channel')
        )
    except ObjectDoesNotExist:
        return

    subtype = event.get('subtype', None)
    if subtype in ignored_subtypes:
        return

    if subtype:
        msg = {
            'user': event.get('previous_message').get('user'),
            'ts': event.get('previous_message').get('ts'),
            'text': event.get('message', {}).get('text', None)
        }
    else:
        msg = {
            'user': event.get('user'),
            'ts': event.get('ts'),
            'text': event.get('text')
        }

    user, created = User.objects.get_or_create(
        api_id=msg.get('user')
    )

    thread_ts = event.get('thread_ts', None) or \
        event.get('message', {}).get('thread_ts', None)
    # If thread
    if thread_ts and (
        event.get('parent_user_id', None) or
        event.get('message', {}).get('parent_user_id', None)
    ):
        if not channel.chat_type.kwargs_in_threads:
            return

        try:
            key, value = msg.get('text').split(': ', 1)
        except ValueError:
            # Don't handle threads that don't follow key-value pattern
            return

        try:
            original_message = Message.objects.get(
                timestamp=strptimestamp(thread_ts)
            )
        except ObjectDoesNotExist:
            raise MessageNotFoundError(
                '{}'.format(strptimestamp(thread_ts))
            )
        KeywordArgument.objects.update_or_create(
            timestamp=strptimestamp(msg.get('ts')),
            message=original_message,
            user=user,
            defaults={
                "key": key,
                "value": value
            }
        )
    else:
        message, created = Message.objects.update_or_create(
            channel=channel,
            timestamp=strptimestamp(msg.get('ts')),
            user=user,
            defaults={
                'text': marker.mark(msg.get('text'))
            }
        )

        # Attachments
        attachments = event.get('message', {}).get('attachments')
        if attachments:
            for attachment in attachments:
                handle_attachment(message.pk, attachment)



from rest_framework import viewsets

from .models import Channel, ChatType
from .serializers import (ChannelListSerializer, ChannelSerializer,
                          ChatTypeSerializer)


class ChannelViewset(viewsets.ReadOnlyModelViewSet):
    queryset = Channel.objects.all()
    serializer_class = ChannelSerializer
    lookup_field = 'pk'
    authentication_classes = []
    permission_classes = []
    pagination_class = None

    def get_serializer_class(self):
        if hasattr(self, 'action') and self.action == 'list':
            return ChannelListSerializer
        return ChannelSerializer

    def get_queryset(self):
        queryset = Channel.objects.all()
        chat_type = self.request.query_params.get('chat_type', None)
        if chat_type:
            queryset = queryset.filter(chat_type__name=chat_type)
        return queryset


class ChatTypeViewset(viewsets.ReadOnlyModelViewSet):
    queryset = ChatType.objects.all()
    serializer_class = ChatTypeSerializer
    authentication_classes = []
    permission_classes = []
    pagination_class = None



import os

from setuptools import find_packages, setup

import slackchat

os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name=slackchat.__package__,
    version=slackchat.__version__,
    packages=find_packages(exclude=('example', 'docs',)),
    include_package_data=True,
    url='https://github.com/The-Politico/django-slackchat-serializer',
    license='MIT',
    description='A Django app that serializes conversations in Slack.',
    python_requires='>=3',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    install_requires=[
        'celery',
        'djangorestframework',
        'emoji',
        'markdown',
        'markslack',
        'Pillow',
        'slackclient',
        'slacker',
        'tqdm',
    ]
)



# Scandalously stolen from https://github.com/onepill/django-simplemde
from django.contrib.admin import widgets as admin_widgets
from django.db.models import TextField

from .widgets import MarkdownEditorWidget


class MarkdownField(TextField):
    def __init__(self, *args, **kwargs):
        self.widget = MarkdownEditorWidget()
        super(MarkdownField, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        defaults = {'widget': self.widget}
        defaults.update(kwargs)

        if defaults['widget'] == admin_widgets.AdminTextareaWidget:
            defaults['widget'] = self.widget
        return super(MarkdownField, self).formfield(**defaults)



import re
import uuid
from datetime import datetime
from urllib.parse import urljoin

from django.db import models
from django.urls import reverse
from django.utils import timezone
from django.utils.encoding import escape_uri_path
from django.utils.safestring import mark_safe
from markdown import markdown
from slackchat.conf import settings
from slackchat.fields import MarkdownField


class Channel(models.Model):
    """
    A Slack channel that hosts a slackchat.
    """
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)

    api_id = models.SlugField(
        max_length=10, null=True, blank=True, editable=False,
        help_text="Slack API channel ID"
    )

    team_id = models.SlugField(
        max_length=10, null=True, blank=True, editable=False,
        help_text="Slack API team ID"
    )

    chat_type = models.ForeignKey(
        'ChatType', on_delete=models.PROTECT)

    owner = models.ForeignKey('slackchat.User', on_delete=models.PROTECT)

    title = models.CharField(max_length=150, blank=True, null=True)

    introduction = MarkdownField(
        blank=True, null=True,
        help_text="Some introductory paragraph text (in markdown syntax).")

    meta_title = models.CharField(
        max_length=300, blank=True, null=True,
        help_text="Page title")
    meta_description = models.CharField(
        max_length=300, blank=True, null=True,
        help_text="Page description")
    meta_image = models.URLField(
        blank=True, null=True,
        help_text="Share image URL")

    publish_path = models.CharField(
        max_length=300, blank=True, unique=True,
        help_text="A relative path you can use when \
        publishing the slackchat, e.g., \
        <span style='color:grey; font-weight:bold;'>/2018-02-12/econ-chat/\
        </span>."
    )

    publish_time = models.DateTimeField(
        default=timezone.now, help_text="Dateline.")

    live = models.BooleanField(
        default=False, help_text="Determines whether page should re-poll for new messages \
        while chat is live.")

    @property
    def published_link(self):
        if settings.PUBLISH_ROOT:
            relative_path = self.publish_path.lstrip('/')
            link = urljoin(settings.PUBLISH_ROOT, relative_path)
            return mark_safe(
                '<a href="{0}" target="_blank">{0}</a>'.format(link))
        else:
            return self.publish_path

    @property
    def api_link(self):
        link = reverse('slackchat-channel-detail', args=[self.id])
        return mark_safe(
            '<a href="{0}" target="_blank">{0}</a>'.format(link))

    @property
    def slack_link(self):
        if self.team_id:
            return mark_safe(
                '<a href="slack://channel?id={0}&team={1}"\
                 target="_blank">{2}</a>'.format(
                    self.api_id,
                    self.team_id,
                    self.slackchat
                ))
        else:
            return '-'

    @property
    def slackchat(self):
        return 'slackchat-{}'.format(self.id.hex[:10])

    def save(self, *args, **kwargs):
        if self.publish_path:
            self.publish_path = escape_uri_path(self.publish_path)
        else:
            self.publish_path = '/{}/{}/'.format(
                datetime.today().strftime('%Y-%m-%d'),
                self.id.hex[:8]
            )
        super().save(*args, **kwargs)

    def get_introduction(self):
        if self.chat_type.render_to_html:
            return mark_safe(markdown(self.introduction))
        return self.introduction

    def __str__(self):
        return self.slackchat



from django.contrib import admin

from .models import (Argument, Attachment, Channel, ChatType,
                     CustomContentTemplate, KeywordArgument, Message, Reaction,
                     User, Webhook)


class ChannelAdmin(admin.ModelAdmin):
    readonly_fields = ('api_id', 'team_id',)
    fieldsets = (
        (None, {
            'fields': ('api_id', 'team_id', 'chat_type', 'owner')
        }),
        ('Publishing info', {
            'fields': ('publish_path', 'publish_time', 'live')
        }),
        ('Page content', {
            'fields': ('title', 'introduction')
        }),
        ('SEO tags', {
            'fields': ('meta_title', 'meta_description', 'meta_image')
        })
    )
    list_display = (
        'slackchat',
        'published_link',
        'api_link',
        'slack_link',
        'live',
    )
    autocomplete_fields = ['owner']


class UserAdmin(admin.ModelAdmin):
    search_fields = ['first_name', 'last_name']


class WebhookAdmin(admin.ModelAdmin):
    fields = ('endpoint', 'verified',)


admin.site.register(Argument)
admin.site.register(Attachment)
admin.site.register(Channel, ChannelAdmin)
admin.site.register(User, UserAdmin)
admin.site.register(CustomContentTemplate)
admin.site.register(Message)
admin.site.register(Reaction)
admin.site.register(KeywordArgument)
admin.site.register(ChatType)
admin.site.register(Webhook, WebhookAdmin)



import re
import uuid

import requests

from celery import shared_task
from slackchat.conf import settings
from slackchat.models import Webhook


@shared_task(acks_late=True)
def post_webhook(channel_id):
    data = {
        "token": settings.WEBHOOK_VERIFICATION_TOKEN,
        "type": "update_notification",
        "channel": channel_id,
    }
    for webhook in Webhook.objects.all():
        if webhook.verified:
            requests.post(webhook.endpoint, json=data)


def clean_response(response):
    """ Cleans string quoting in response. """
    response = re.sub('^[\'"]', '', response)
    response = re.sub('[\'"]$', '', response)
    return response


@shared_task(acks_late=True)
def verify_webhook(pk):
    webhook = Webhook.objects.get(pk=pk)
    if not webhook.verified:
        challenge = uuid.uuid4().hex[:10]
        response = requests.post(webhook.endpoint, json={
            "token": settings.WEBHOOK_VERIFICATION_TOKEN,
            "type": "url_verification",
            "challenge": challenge,
        })
        if response.status_code == requests.codes.ok:
            if clean_response(response.text) == challenge:
                webhook.verified = True
                webhook.save()
